///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Intel Corporation
// TU-Munich, S.Herrmann
// Heinrich-Hertz-Institute (HHI), Bela Makai, Karsten Mueller
// (contributing organizations names)

//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties
// from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  ScalablrColor.cpp
//
#include <vector>
#include <stdio.h>
#include "Descriptors.h"
#include "readparam.h"

using namespace XM;

const UUID ScalableColorDescriptorInterface::myID = UUID("09df7000-db15-11d2-affe-0080c7e1e76d");
const char *ScalableColorDescriptorInterface::myName = "ScalableColorDescriptionInterface";

const UUID ScalableColorDescriptor::myID = UUID("def04360-dafb-11d2-affe-0080c7e1e76d");
const char *ScalableColorDescriptor::myName = "ScalableColorDescriptor";

const UUID ScalableColorDescriptor::valID = UUID("f241a300-dafb-11d2-affe-0080c7e1e76d");
const char *ScalableColorDescriptor::valName = "";

const UUID ScalableColorDescriptorInterfaceABC::myID = UUID();

ScalableColorDescriptorInterfaceABC::ScalableColorDescriptorInterfaceABC()
{ SetInterfaceID(myID);}

//=============================================================================
ScalableColorDescriptorInterface::ScalableColorDescriptorInterface(ScalableColorDescriptor *aHisto):
m_ScalableColorDescriptor(aHisto)
{
	SetInterfaceID(myID);
}

//----------------------------------------------------------------------------
ScalableColorDescriptorInterface::~ScalableColorDescriptorInterface()
{
}

//----------------------------------------------------------------------------
void ScalableColorDescriptorInterface::addref(void)
{ m_ScalableColorDescriptor->addref(); }

//----------------------------------------------------------------------------
void ScalableColorDescriptorInterface::release(void)
{ m_ScalableColorDescriptor->release(); }

//----------------------------------------------------------------------------
void ScalableColorDescriptorInterface::SetValuesFromParameterFile(void)

{

	m_ScalableColorDescriptor->SetNumberOfBitplanesDiscarded(
		atoi(getpara("GetNumberOfBitplanesDiscarded","0")));
}

//----------------------------------------------------------------------------
unsigned long *ScalableColorDescriptorInterface::GetScalableHistogram(void)
{
	return m_ScalableColorDescriptor->GetScalableHistogram();
}

//----------------------------------------------------------------------------
unsigned long ScalableColorDescriptorInterface::GetNumberOfCoefficients(void)
{
	return m_ScalableColorDescriptor->GetNumberOfCoefficients();
}

//----------------------------------------------------------------------------
unsigned long ScalableColorDescriptorInterface::SetNumberOfCoefficients(unsigned long size)	
{
	return m_ScalableColorDescriptor->SetNumberOfCoefficients(size);
}

//----------------------------------------------------------------------------
unsigned long ScalableColorDescriptorInterface::GetNumberOfBitplanesDiscarded(void)
{
	return m_ScalableColorDescriptor->GetNumberOfBitplanesDiscarded();
}

//----------------------------------------------------------------------------
unsigned long
ScalableColorDescriptorInterface::SetNumberOfBitplanesDiscarded(unsigned long factor)
{
	return m_ScalableColorDescriptor->SetNumberOfBitplanesDiscarded(factor);
}

//----------------------------------------------------------------------------
unsigned long ScalableColorDescriptorInterface::SetCoefficient(unsigned long index, int value)
{
	return m_ScalableColorDescriptor->SetCoefficient(index, value);
}

//----------------------------------------------------------------------------
unsigned long ScalableColorDescriptorInterface::GetCoefficient(unsigned long index)
{
	return m_ScalableColorDescriptor->GetCoefficient(index);
}

//----------------------------------------------------------------------------
const UUID& ScalableColorDescriptorInterface::GetInterfaceID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *ScalableColorDescriptorInterface::GetName(void)
{ return myName; }


//-----------------------------------------------------------------------------
unsigned long ScalableColorDescriptorInterface::ExportDDL(GenericDSInterfaceABC 
																													*aParentDescription)
{
  return m_ScalableColorDescriptor->ExportDDL(aParentDescription);
}

//-----------------------------------------------------------------------------
unsigned long ScalableColorDescriptorInterface::ImportDDL(GenericDSInterfaceABC 
																													*aDescription)
{
  return m_ScalableColorDescriptor->ImportDDL(aDescription);
}

//=============================================================================
ScalableColorDescriptor::ScalableColorDescriptor():
m_refcount(0),
m_Interface(this),
m_isProprietary(false),
m_ScalableHistogram(0),
m_NumberOfCoefficients(0),
m_NumberOfBitplanesDiscarded(0)
{
	SetExposedInterface(&m_Interface);
}

//----------------------------------------------------------------------------
ScalableColorDescriptor::~ScalableColorDescriptor()
{

	if (m_ScalableHistogram) delete [] m_ScalableHistogram;

#ifdef VERBOSE
       fprintf(stderr,"ScalableColor delete\n");
#endif
}

//----------------------------------------------------------------------------
const UUID& ScalableColorDescriptor::GetObjectID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *ScalableColorDescriptor::GetName(void)
{ return myName; }

//----------------------------------------------------------------------------
const UUID& ScalableColorDescriptor::GetValueID(void)
{ return valID; }

//----------------------------------------------------------------------------
const char* ScalableColorDescriptor::GetValueName(void)
{ return valName; }

//----------------------------------------------------------------------------
bool ScalableColorDescriptor::IsProprietary(void)
{ return m_isProprietary; }

//----------------------------------------------------------------------------
void ScalableColorDescriptor::addref(void)

{
#ifdef VERBOSE
       fprintf(stderr,"ScalableColor connect\n");
#endif
       m_refcount++;
}

//----------------------------------------------------------------------------
void ScalableColorDescriptor::release(void)

{
#ifdef VERBOSE
       fprintf(stderr,"ScalableColor release\n");
#endif
       if (!(--m_refcount)) delete this;
}


//----------------------------------------------------------------------------
unsigned long* ScalableColorDescriptor::GetScalableHistogram(void)
{
	return m_ScalableHistogram;
}

//----------------------------------------------------------------------------
unsigned long ScalableColorDescriptor::GetNumberOfCoefficients(void)
{
	return m_NumberOfCoefficients;
}

//----------------------------------------------------------------------------
unsigned long ScalableColorDescriptor::SetNumberOfCoefficients(unsigned long 
																															 newSize)
{
	if (m_NumberOfCoefficients == newSize) return 0;

	if (m_ScalableHistogram) delete [] m_ScalableHistogram;
	m_ScalableHistogram = new unsigned long [newSize];

	m_NumberOfCoefficients = newSize;

	return 0;
}

//----------------------------------------------------------------------------
unsigned long ScalableColorDescriptor::
   SetNumberOfBitplanesDiscarded(unsigned long aRenorm)
{
	m_NumberOfBitplanesDiscarded = aRenorm;

	return 0;
}

//----------------------------------------------------------------------------
unsigned long ScalableColorDescriptor::GetNumberOfBitplanesDiscarded()
{
	return m_NumberOfBitplanesDiscarded;
}

//----------------------------------------------------------------------------
unsigned long ScalableColorDescriptor::SetCoefficient(unsigned long index, 
																											int value)
{
	if (index < 0 || index >= m_NumberOfCoefficients)
		return (unsigned long)-1;
	m_ScalableHistogram[index] = value;

	return 0;
}

//----------------------------------------------------------------------------
unsigned long ScalableColorDescriptor::GetCoefficient(unsigned long index)
{
	if (index < 0 || index >= m_NumberOfCoefficients)
		return (unsigned long)-1;

	return m_ScalableHistogram[index];
}


//-----------------------------------------------------------------------------
unsigned long ScalableColorDescriptor::ExportDDL(GenericDSInterfaceABC *aParentDescription)
{
  if (!aParentDescription)
    return (unsigned long)-1;

  GenericDS l_DDLDescription;
  l_DDLDescription = aParentDescription->CreateChild("Descriptor");
  l_DDLDescription.SetAttribute("xsi:type", "ScalableColorType");
//////////////////////////

	int NumberOfCoefficients=atoi(getpara("NumberOfCoefficients","256"));
	
	if((NumberOfCoefficients!=16)&&(NumberOfCoefficients!=32)&&
	   (NumberOfCoefficients!=64)&&(NumberOfCoefficients!=128)&&
		 (NumberOfCoefficients!=256))
	return (unsigned long)-1;

	int anzkoeff = 0;
	//if(NumberOfCoefficients==16) anzkoeff=0;
	if(NumberOfCoefficients==32) anzkoeff=1;
	if(NumberOfCoefficients==64) anzkoeff=2;
	if(NumberOfCoefficients==128) anzkoeff=3;
	if(NumberOfCoefficients==256) anzkoeff=4;
  l_DDLDescription.SetAttribute("NumberOfCoefficients", anzkoeff);
	
	/*put NumberOfRedBits*/
	int NumberOfBitplanesDiscarded = GetNumberOfBitplanesDiscarded();
	int anzredbits = -1;
	if((NumberOfBitplanesDiscarded>=0)&&(NumberOfBitplanesDiscarded<5))
		anzredbits=NumberOfBitplanesDiscarded;
	if(NumberOfBitplanesDiscarded==5) anzredbits=6;
	if(NumberOfBitplanesDiscarded==6) anzredbits=5;
	if(NumberOfBitplanesDiscarded>6) anzredbits=6;
  l_DDLDescription.SetAttribute("NumberOfBitplanesDiscarded", anzredbits);
	
	
  GenericDS Coeff_element = l_DDLDescription.CreateChild("Coefficients");

	vector<int> element(NumberOfCoefficients);
	for (int i=0; i<NumberOfCoefficients; i++) 
	{
		int no_of_bits=XM::scalableColorQuantValues[i][1]-1-NumberOfBitplanesDiscarded;

		if(no_of_bits<=0) no_of_bits = 1;

		element[i] = GetCoefficient(i);// >> (sizeof(long) - no_of_bits)		
	}

	Coeff_element.SetValue(element);

  return 0;
}

//-----------------------------------------------------------------------------
unsigned long ScalableColorDescriptor::ImportDDL(GenericDSInterfaceABC 
																								 *aDescription)
{
  if (!aDescription)
    return (unsigned long)-1;

  GenericDS l_DDLDescription;
  GenericDSInterfaceABC *l_DDLDescriptionInterface = NULL;

  string xsitype;
  if (aDescription->GetDSName() == "Descriptor")
  {
    aDescription->GetTextAttribute("xsi:type", xsitype);
    if (xsitype == "ScalableColorType")
    {
      l_DDLDescriptionInterface = aDescription;
    }
  }

  if (!l_DDLDescriptionInterface)
  {
    l_DDLDescription = aDescription->GetDescription("Descriptor");

    while (!l_DDLDescription.isNull())
    {
      l_DDLDescription.GetTextAttribute("xsi:type", xsitype);
      if (xsitype == "ScalableColorType")
        break;
      l_DDLDescription = l_DDLDescription.GetNextSibling("Descriptor");
    }

    if (l_DDLDescription.isNull())
      return (unsigned long)-1;

    l_DDLDescriptionInterface = l_DDLDescription.GetInterface();
  }

	int anzkoeff;
  l_DDLDescriptionInterface->GetIntAttribute("NumberOfCoefficients", 
		anzkoeff);
	
	int NumberOfCoefficients=-1;
	if(anzkoeff==0) NumberOfCoefficients=16;
	if(anzkoeff==1) NumberOfCoefficients=32;
	if(anzkoeff==2) NumberOfCoefficients=64;
	if(anzkoeff==3) NumberOfCoefficients=128;
	if(anzkoeff==4) NumberOfCoefficients=256;
	if (!NumberOfCoefficients)
		return 0;
	SetNumberOfCoefficients(NumberOfCoefficients); 
	
	/*get normalization*/
	int anzredbits;
  l_DDLDescriptionInterface->GetIntAttribute("NumberOfBitplanesDiscarded", 
		anzredbits);
	
	int NumberOfBitplanesDiscarded=-1;
	if(anzredbits==0) NumberOfBitplanesDiscarded=0;
	if(anzredbits==1) NumberOfBitplanesDiscarded=1;
	if(anzredbits==2) NumberOfBitplanesDiscarded=2;
	if(anzredbits==3) NumberOfBitplanesDiscarded=3;
	if(anzredbits==4) NumberOfBitplanesDiscarded=4;
	if(anzredbits==5) NumberOfBitplanesDiscarded=6;
	if(NumberOfBitplanesDiscarded==-1) NumberOfBitplanesDiscarded=8;
		
	SetNumberOfBitplanesDiscarded(NumberOfBitplanesDiscarded);
		
	

  GenericDS Coeff_element = l_DDLDescriptionInterface->
		GetDescription("Coefficients");
  vector<int> element;
  Coeff_element.GetIntVector(element);
	
	for (int i=0; i<NumberOfCoefficients; i++) 
	{
//		int no_of_bits = 
//			XM::scalableColorQuantValues[i][1]-1-NumberOfBitplanesDiscarded;
		int coeff = element[i]; // << (sizeof(long) - no_of_bits)
		SetCoefficient(i,coeff);
				
	}

  return 0;
}

//----------------------------------------------------------------------------
ScalableColorDescriptorInterfaceABC *ScalableColorDescriptor::
  GetInterface(void)
{ return &m_Interface; }



