///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Stephan Herrmann
// Munich University of Technology
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third
// parties from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  MultiImageSearch.h
//

#ifndef __MULTIIMAGESEARCH_H__
#define __MULTIIMAGESEARCH_H__

#include "XMInterfaces.h"                            /* Naming Convention */
#include "XMObjects.h"                               /* Naming Convention */
#include "Descriptors.h"
#include "DescriptionSchemes.h"

//=============================================================================

namespace XM
{


// Forward Declarations:
class MultiImageSearchTool;

//=============================================================================
class MultiImageSearchInterfaceABC: public I_SearchInterfaceABC
{
public:
  MultiImageSearchInterfaceABC();
  virtual ~MultiImageSearchInterfaceABC() {};

  virtual void destroy()=0;

  virtual int SetRefDescriptorInterface
    (MultiImageDescriptorInterfaceABC
     *aMultiImageDescriptorInterface) = 0;
  virtual MultiImageDescriptorInterfaceABC*
    GetQueryDescriptorInterface(void) = 0;
  virtual int SetQueryDescriptorInterface
    (MultiImageDescriptorInterfaceABC
     *aMultiImageDescriptorInterface) = 0;

  virtual double GetDistance(void) = 0;
	
  static const UUID myID;
};

//=============================================================================
class MultiImageSearchInterface: 
public MultiImageSearchInterfaceABC
{
public:
  MultiImageSearchInterface(MultiImageSearchTool* aTool);
  virtual ~MultiImageSearchInterface();
	
  virtual const UUID& GetInterfaceID(void);
  virtual const char *GetName(void);

  virtual void destroy();

  virtual int SetRefDescriptorInterface
    (MultiImageDescriptorInterfaceABC
     *aMultiImageDescriptorInterface);
  virtual MultiImageDescriptorInterfaceABC*
    GetQueryDescriptorInterface(void);
  virtual int SetQueryDescriptorInterface
    (MultiImageDescriptorInterfaceABC
     *aMultiImageDescriptorInterface);

  virtual double GetDistance(void);


  static const UUID myID;
  static const char *myName;

private:
  MultiImageSearchTool *m_SearchTool;
};

//=============================================================================
class MultiImageSearchTool: public Search
{
friend MultiImageSearchInterface;
public:
  MultiImageSearchTool();
  MultiImageSearchTool(MultiImageDescriptorInterfaceABC
		      *aQueryDescriptorInterface);

  virtual const UUID& GetObjectID(void);
  virtual const char *GetName(void);

  virtual MultiImageSearchInterfaceABC *GetInterface(void);

  virtual bool SupportsPush(void);
  virtual bool SupportsPull(void);

  virtual int SetRefDescriptorInterface
    (MultiImageDescriptorInterfaceABC
     *aMultiImageDescriptorInterface);
  virtual MultiImageDescriptorInterfaceABC*
    GetQueryDescriptorInterface(void);
  virtual int SetQueryDescriptorInterface
    (MultiImageDescriptorInterfaceABC
     *aMultiImageDescriptorInterface);

  virtual double GetDistance(void);

  static const UUID myID;
  static const char * myName;
private:
  virtual ~MultiImageSearchTool();

  MultiImageSearchInterface m_Interface;
  MultiImageDescriptorInterfaceABC *m_RefDescriptorInterface;
  MultiImageDescriptorInterfaceABC *m_QueryDescriptorInterface;
  ScalableColorSearchInterfaceABC *m_ScalableColorSearch;
  ContourShapeSearchInterfaceABC *m_ContourShapeSearch;
  float m_ScalableColorScale;
  float m_ScalableColorWheight;
  float m_ContourShapeScale;
  float m_ContourShapeWheight;
};

};

#endif //__MULTIIMAGESEARCH_H__
