///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Yong-Sung Kim - Hanyang University, Korea
// (contributing organizations names)
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties
// from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-2000.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  RegionShapeSearch.h
//
#ifndef __REGION_SHAPE_SEARCH_H__
#define __REGION_SHAPE_SEARCH_H__

#include "XMInterfaces.h"                            /* Naming Convention */
#include "XMObjects.h"                               /* Naming Convention */
#include "MatchList.h"
#include "Descriptors.h"

//=============================================================================

namespace XM
{

// Forward Declarations:
class RegionShapeSearchTool;

//=============================================================================
class RegionShapeSearchInterfaceABC: public I_SearchInterfaceABC
{
public:
	RegionShapeSearchInterfaceABC();
	virtual ~RegionShapeSearchInterfaceABC() {};

	virtual void destroy() = 0;

	virtual int SetRefDescriptorInterface
	  (RegionShapeDescriptorInterfaceABC
	   *aRegionShapeDescriptorInterface) = 0;
	virtual int SetQueryDescriptorInterface
	  (RegionShapeDescriptorInterfaceABC
	   *aRegionShapeDescriptorInterface) = 0;
	virtual double GetDistance(void) = 0;

	static const UUID myID;
};

//=============================================================================
class RegionShapeSearchInterface: 
  public RegionShapeSearchInterfaceABC
{
public:
	RegionShapeSearchInterface(RegionShapeSearchTool* aTool);
	virtual ~RegionShapeSearchInterface();
	
	virtual const UUID& GetInterfaceID(void);
	virtual const char *GetName(void);

	virtual void destroy();

	virtual int SetRefDescriptorInterface
	  (RegionShapeDescriptorInterfaceABC
	   *aRegionShapeDescriptorInterface);
	virtual int SetQueryDescriptorInterface
	  (RegionShapeDescriptorInterfaceABC
	   *aRegionShapeDescriptorInterface);
	virtual double GetDistance(void);


	static const UUID myID;
	static const char *myName;

private:
	RegionShapeSearchTool *m_SearchTool;
};

//=============================================================================
class RegionShapeSearchTool: public Search
{
friend RegionShapeSearchInterface;
public:
	RegionShapeSearchTool();
	RegionShapeSearchTool(RegionShapeDescriptorInterfaceABC
                               *aQueryDescriptorInterface);

	virtual const UUID& GetObjectID(void);
	virtual const char *GetName(void);
	virtual RegionShapeSearchInterfaceABC *GetInterface(void);

	virtual bool SupportsPush(void);
	virtual bool SupportsPull(void);

	virtual int SetRefDescriptorInterface
	  (RegionShapeDescriptorInterfaceABC
	   *aRegionShapeDescriptorInterface);
	virtual int SetQueryDescriptorInterface
	  (RegionShapeDescriptorInterfaceABC
	   *aRegionShapeDescriptorInterface);
	virtual double GetDistance(void);

	static const UUID myID;
	static const char * myName;
private:
	virtual ~RegionShapeSearchTool();

	RegionShapeSearchInterface m_Interface;
	RegionShapeDescriptorInterfaceABC *m_RefDescriptorInterface;
	RegionShapeDescriptorInterfaceABC *m_QueryDescriptorInterface;
};

};


#endif //__REGION_SHAPE_SEARCH_H__
